SKG:SetCommPrefix( SKG_CHANNEL_PREFIX )
SKG:SetDefaultCommPriority( "NORMAL" )

SKG:AddCommListener( SKG_CHANNEL_PREFIX, "GUILD" )		-- just about everything else
SKG:AddCommListener( SKG_CHANNEL_PREFIX, "WHISPER" )	-- requests for bidding/syncing
SKG:AddCommListener( SKG_CHANNEL_PREFIX, "GROUP" )		-- Raid looting

-- Since I'm going to tokenize the first field of any message as the "Command token",
-- break apart the receiving handler by those tokens
-- For a summary of command tokens, see Documentation/EventTokens.lua

SKG.OnCommReceive = {}

SKG.OnCommReceive.DEBUG = function ( self, prefix, distribtion, sender, message )
	SKG_Debug( "Comm: Debug (" .. sender .. ") : " .. message )
end

------------------------------------------------
-- Syncronization Commands 
------------------------------------------------

SKG.OnCommReceive.SYNSS = function ( self, prefix, distribtion, sender, message )
	-- SYNSS
	-- Payload: { TID, ListName, ListHash }
	-- Description: Snapshot the current lists and wipe our Log

	-- Details:
	-- SYNSS events are sent to "catalog" a save point for lists once the syncers
	-- agree they all have a correct list. It can also be done to reduce the space
	-- of the log is taking up.
	--
	-- General logic: if your hash for the named list is the same as the one specified
	-- in the payload, you save a copy of the named list to the Snapshot structure
	-- and intialize the Log for that list to the snapshot event (clearing it out)
	--
	-- If you do NOT have the same hash for the named list, flag an error to the user
	-- indicating that they will probably need to do a forced sync from someone to
	-- get the correct list. (it probably means they don't have the right list).

	-- If the receiving client is read-only, this can be safely ignored.

	SKG_Debug( "List Snapshot received" )
	if not ( SKG_SECURITY_LEVEL == "U" ) then
		-- We're not read-only, do that thang.
		local TID, ListName, ListHash = unpack( message, 1, 3 )
		local db = SKG.db.profile

		-- Check the hash. If our hash tastes like their hash, commit
		-- need to check the has using latestTID, not the TID sent
		local latestTID = SKG_GetLatestTID( db.Log[ListName] )
		local LH, SH = SKG_HashRollList( latestTID, ListName )
		if ( LH == ListHash ) then
			-- do eeet
			db.Snapshot[ListName] = {}
			db.Snapshot[ListName] = db.RollLists[ListName]
			db.Log[ListName] = {}
			db.SnapshotLM[ListName] = {}
			db.SnapshotLM[ListName] = db.LastMod[ListName]
			SKG_AddLogEntry( TID, ListName, "SYNSS", ListHash, ListHash )
			SKG_Print( SKGL["SKG_MSG_SNAPSHOTCOMPLETE"] )
		else
			-- Popup some warning that the lists are not synced, and they should
			-- get a sync from someone.
			StaticPopup_Show( "SKG_SYNSS_WITH_BAD_HASH" )
		end
	end
end


SKG.OnCommReceive.SYNRK = function ( self, prefix, distribtion, sender, message )
	-- SYNRK
	-- Payload: List
	-- Description: Request Sync Key for "List" (reply to sender)

	SKG_Debug( "Comm: Sync Key Request received" )
	local db = SKG.db.profile

	-- TODO: Security: Put in a check to see if we're a syncer. If not, don't do anything
	if ( message == "Players" ) then
		local longHash, shortHash = SKG_HashRollList( 0, "Players" )
		SKG:SendCommMessage( "WHISPER", sender, "SYNSK", longHash .. shortHash )
	else
		local latestTID = 0
		local longHash, shortHash = "", ""
		-- Make sure we have that list!!
		if ( db.RollLists[message] ) then
			latestTID = SKG_GetLatestTID( db.Log[message] )
			longHash, shortHash = SKG_HashRollList( latestTID, message )
		else
			shortHash = "nil"
		end
		SKG:SendCommMessage( "WHISPER", sender, "SYNSK", shortHash )
	end
end

SKG.OnCommReceive.SYNSK = function ( self, prefix, distribtion, sender, message )
	-- SYNSK
	-- Payload: List
	-- Description: Send Sync Key for "List"

	SKG_Debug( "Comm: Sync Key Send received" )

	-- Don't do anything if I'm the sender of the request
	if not ( sender == SKG.me ) then
		SKG_currentSyncItems = SKG_currentSyncItems + 1
		local currButton = SKG_currentSyncItems

		SKG_SYF_NameListItem[currButton]:SetText( sender )
		SKG_SYF_NameListItem[currButton]:Show()
		SKG_SYF_NameListItem[currButton]:Enable()
		SKG_SYF_HashListItem[currButton]:SetText( message )
		SKG_SYF_HashListItem[currButton]:Show()
	end
end

SKG.OnCommReceive.SYNRL = function ( self, prefix, distribtion, sender, message )
	-- SYNRL
	-- Payload: { List, ListHash }
	-- Description: Request whispered from "sender" to send them your copy of
	--		"List". This request requires no acknowledgement. Simply send them
	--		the requested list if you have it. The ListHash is sent to compare
	--		to our own. If the ListHash matches, there's no work to be done.

	SKG_Debug( "Comm: Sync List Request" )

	-- I can't send myself this request, so don't bother checking...
	-- don't bother recording requests, since they don't change any data
	-- just send over the list

	local listName, listHash = unpack( message, 1, 2 )
	local db = SKG.db.profile

	if ( listName == "Players" ) then
		SKG_Debug( "SYNRL: list is Players" )
		if ( next( db.Players ) == nil ) then
			SKG_Debug( "SYNRL: seems Players is empty??" )
			-- I don't think I want to send nil. Just don't do anything for now
			-- SKG:SendCommMessage( "WHISPER", sender, "SYNSL", nil )
		else
			-- don't bother comparing listhashes for Players. Just send it
			SKG_Debug( "SYNRL: Players is not empty" )
			local newTID = SKG_CreateTID( 0, 0, 0 )
			SKG_Print( string.format( SKGL["SKG_MSG_SENDINGLIST"], "Players", sender ) )
			SKG:SendCommMessage( "WHISPER", sender, "SYNSL", 
				{ newTID, "Players", db.Players, db.Alts } )
		end
	else
		-- otherwise, it should be a rolllist
		-- do we have that rolllist?
		SKG_Debug( "SYNRL: list is not Players" )
		if ( db.RollLists[listName] ) then
			-- ok, we have the same list. Does our hash match his hash?
			SKG_Debug( "SYNRL: we have that list" )
			local maxTID = SKG_GetLatestTID( db.Log[listName] )
			local ourHash = SKG_HashRollList( maxTID, listName )
			if ( ourHash == listHash ) then
				SKG_Debug( "SYNRL: hashes same" )
			else
				-- Send along the list and its log
				SKG_Debug( "SYNRL: hashes not same" )
				local playerHash = SKG_HashRollList( 0, "Players" )
				SKG_Debug( "SYNRL: created playerHash" )
				local newTID = SKG_CreateTID( 0, 0, 0 )
				SKG_Debug( "SYNRL: created newTID" )
				SKG_Print( string.format( SKGL["SKG_MSG_SENDINGLIST"], listName, sender ) )
				SKG:SendCommMessage( "WHISPER", sender, "SYNSL", 
					{ newTID, listName, db.RollLists[listName],
						db.Log[listName], db.LastMod[listName], 
						db.Snapshot[listName], db.SnapshotLM[listName],
						playerHash }
				)
				SKG_Debug( "SYNRL: sent message SYNSL" )
			end
		end
	end
end

SKG.OnCommReceive.SYNSL = function ( self, prefix, distribtion, sender, message )
	-- SYNSL
	-- Payload: { TID, listName, Table1, Table2, [ PlayerHash ] }
	-- Description: This can only be received by someone if they had previously
	--		requested it, so there's no need to send any authorization requests.
	--		This will be returned in one of two ways:
	--			{ TID, "Players", PlayersTable, AltsTable }
	--			{ TID, listName, listTable, listLogTable, 
	--				LMTable, snapTable, snapLMTable, playerHash }

	SKG_Debug( "Comm: Received Sent List (answer from SYNRL)" )

	local db = SKG.db.profile
	local TID, listName = unpack( message, 1, 2 )
	if ( listName == "Players" ) then
		SKG_Debug( "SYNSL: Players list" )
		local playerList, altList = unpack( message, 3, 4 )

		local newPL = db.Players
		for hash, plInfo in pairs( playerList ) do
			if ( not newPL[hash] ) then
				-- we got a new one, add it in.
				newPL[hash] = plInfo
				SKG_Debug( "Adding " .. hash .. " (" .. newPL[hash].name .. "/" .. newPL[hash].class .. ")" )
			end
		end
		local lHash, sHash = SKG_HashRollList( 0, "Players" )

		local newAL = db.Alts
		for hash, plAlts in pairs( altList ) do
			if ( not newAL[hash] ) then
				-- I have a player in the alts list that isn't in mine. Make sure it exists
				newAL[hash] = {}
			end
			for altIndex, altInfo in pairs( plAlts ) do
				-- since the lists could be completely different for the two people,
				-- make sure you look at the whole list to see if the alt already exists.
				
				-- so... is this entry in my list already?
				local notfound = true
				local i = 1
				while ( ( i <= #newAL[hash] ) and ( notfound == true ) ) do
					if ( newAL[hash][i].name == plAlts[altIndex].name ) then
						notfound = false
					end
					i = i + 1
				end
				if ( notfound == true ) then
					tinsert( newAL[hash], plAlts[altIndex] )
				end
			end
		end

		SKG_AddLogEntry( TID, "SKG", "SYNSL", 0, lHash .. sHash )
	else
		SKG_Debug( "SYNSL: not Players" )
		local rollList, listLog, lastMod, snap, snapLM, playerHash = unpack( message, 3, 8 )

		local myPlayerHash = SKG_HashRollList( 0, "Players" )

		-- First step... if our player hash is not the same as theirs, we
		-- should not attempt a sync. That needs to be true first.
		if ( myPlayerHash ~= playerHash ) then
			-- error
			SKG_Print( SKGL["SKG_ERROR_BADPLAYERHASH"] )
			return
		else
			-- next step: find out if both the passed list and ours
			-- have the same starting event (SYNSS)
			local oldestTID = SKG_GetOldestTID( listLog )
			local myOldestTID = SKG_GetOldestTID( db.Log[listName] )
			SKG_Debug( "his = " .. oldestTID )
			SKG_Debug( "mine = " .. myOldestTID )

			if ( oldestTID ~= myOldestTID ) then
				-- player needs to be able to "override" a bad starting
				-- point, and simply accept the other player's data as the
				-- correct data
				SKG.OverrideData = { 
					TID, listName, rollList, listLog, lastMod, snap, snapLM
				}
				StaticPopup_Show( "SKG_OVERRIDE_SYNSS" )
				return
			else
				-- Determine which list to send. If nlHash is 0, send
				-- an empty list. If it is not, we send the Snapshot
				-- list
				local tList = {}
				local lmList = {}
				if ( db.Log[listName][myOldestTID].nlHash ~= 0 ) then
					tList = SKG_tblCopy( db.Snapshot[listName] )
					lmList = SKG_tblCopy( db.SnapshotLM[listName] )
				end

				-- Create a combined list of transactions from both my
				-- and the sent list
				local transList = SKG_tblCopy( db.Log[listName] )

				for logTID, logTab in pairs( listLog ) do
					if ( transList[logTID] == nil ) then
						local tempT = SKG_tblCopy( logTab )
						transList[logTID] = tempT
					end
				end

				SKG_RunTransactions( transList, tList, lmList )
				-- the results will be in tList
				db.RollLists[listName] = tList
				db.Log[listName] = transList
				db.LastMod[listName] = lmList
				-- we still need to do LastMod stuff ???
			end
		end
		-- fix
		local lTID = SKG_GetLatestTID( db.Log[listName] )
		local newlHash = SKG_HashRollList( lTID, listName )
		SKG_AddLogEntry( TID, "SKG", "SYNSL", listName, newlHash )
	end
	SKG_Print( SKGL["SKG_MSG_SYNCCOMPLETE"] )
	SKG_ScrollFrame.selected = 0
	SKG_UpdateSPlayers()
	SKG_UpdateMainsChooser()
	SKG_UpdateToonChooser()
	SKG_UpdateCurrentList()
end

SKG.OnCommReceive.SYNPL = function ( self, prefix, distribtion, sender, message )
	-- SYNPL
	-- Payload: (no payload)
	-- Description: This is the "Ping Lists" request, broadcast to the GUILD
	--		channel on login. The responses should come whispered as SYNAL (Ack Lists)
	--		messages.
	--
	-- Notes: This is to get around the fact that not everyone may have the lists
	--		that others do. This will give them at least the stub lists so they can
	--		then easily sync with others.

	-- Only respond if we're a syncer (rewrite for security)
	
	SKG_Debug( "Comm: SYNPL received" )
	-- Don't bother if I'm the sender
	if ( sender ~= SKG.me ) then
		-- build result table
		local ResultSet = {}
		for k, v in pairs( SKG.db.profile.RollLists ) do
			tinsert( ResultSet, k )
		end

		-- Only send if we ourselves have any lists...
		if ( next( ResultSet ) ~= nil ) then
			SKG:SendCommMessage( "WHISPER", sender, "SYNAL", ResultSet )
			SKG_Debug( "Comm: Sending SYNAL" )
		end
	end
end

SKG.OnCommReceive.SYNAL = function ( self, prefix, distribtion, sender, message )
	-- SYNAL
	-- Payload: { List1, List2, ... }
	-- Description: The "Ack Lists" response to a SYNPL request. Whisper channel
	--		the response back with a list of all RollLists (exclude Players/Alts
	--		lists). 

	SKG_Debug( "Comm: SYNAL received" )
	local db = SKG.db.profile
	local newTID = SKG_CreateTID( 0, 0, 0 )
	SKG_AddLogEntry( newTID, "SKG", "SYNAL", 0, 0 )

	for i = 1, #message do
		currList = message[i]
		if not ( db.RollLists[currList] ) then
			newTID = SKG_CreateTID( i, 0, 0 )

			SKG_Debug( "SYNAL:  New List created : " .. currList )

			db.RollLists[currList] = {}
			SKG_UpdateLists()


			-- And start off the log for our new list
			db.LastMod[currList] = {}
			db.Log[currList] = {}
			db.Snapshot[currList] = {}
			db.SnapshotLM[currList] = {}
			SKG_AddLogEntry( newTID, currList, "SYNSS", 0, 0 )
		end
	end
end

------------------------------------------------
-- Security Commands 
------------------------------------------------

SKG.OnCommReceive.SECUR = function ( self, prefix, distribtion, sender, message )
	-- SECRA
	-- Payload: TID { Security Info Table }
	-- Description: Parse the sent Security Info, and see if my 
	--		permissions have changed. Alter my SV as appropriate.

	SKG_Debug( "Comm:  SECUR packet received" )

	-- This all needs a complete rewrite
end

------------------------------------------------
-- List Commands 
------------------------------------------------

SKG.OnCommReceive.LSTCR = function ( self, prefix, distribtion, sender, message )
	-- LSTCR
	-- Payload: { TID, ListName }
	-- Description: Creates a new list called "ListName"

	-- TODO : Redo security

	local db = SKG.db.profile
	local TID, listName = unpack( message, 1, 2 )

	if ( db.RollLists[listName] ) then
		-- We already have the list. Blah. Don't do anything.
	else
		SKG_Debug( "Comm:  New List created" )

		db.RollLists[listName] = {}
		SKG_UpdateLists()

		SKG_AddLogEntry( TID, "SKG", "LSTCR", listName, 0 )

		-- And start off the log for our new list
		db.LastMod[listName] = {}
		db.Log[listName] = {}
		db.Snapshot[listName] = {}
		db.SnapshotLM[listName] = {}
		SKG_AddLogEntry( TID, listName, "SYNSS", 0, 0 )
	end
end

SKG.OnCommReceive.LSTRM = function ( self, prefix, distribtion, sender, message )
	-- LSTRM
	-- Payload: { TID, ListName	}
	-- Description: Removes the list called "ListName"

	-- TODO: Redo security

	local db = SKG.db.profile
	local TID, listName = unpack( message, 1, 2 )

	if ( db.RollLists[listName] ) then
		SKG_Debug( "Comm:  Delete existing list" )

		db.RollLists[listName] = nil
		db.Log[listName] = nil
		db.LastMod[listName] = nil
		db.Snapshot[listName] = nil
		db.SnapshotLM[listName] = nil
		SKG_UpdateLists()
		SKG_ResetCurrentList()

		SKG_AddLogEntry( TID, "SKG", "LSTRM", listName, 0 )
	end
end

SKG.OnCommReceive.LSTRN = function ( self, prefix, distribtion, sender, message )
	-- LSTRN
	-- Payload: { TID, OldList, NewName }
	-- Description: Renames list "OldList" to "NewName"

	-- TODO : rewrite security

	local db = SKG.db.profile
	local TID, oldList, newName = unpack( message, 1, 3 )

	if ( db.RollLists[oldList] ) then
		SKG_Debug( "Comm:  Delete existing list" )

		db.RollLists[newName] = {}
		db.RollLists[newName] = db.RollLists[oldList]
		db.RollLists[oldList] = nil
		SKG_UpdateLists()
		SKG_ResetCurrentList()

		-- Move the aux lists over as well
		db.Log[newName] = db.Log[oldList]
		db.LastMod[newName] = db.LastMod[oldList]
		db.Snapshot[newName] = db.Snapshot[oldList]
		db.SnapshotLM[newName] = db.SnapshotLM[oldList]

		db.Log[oldList] = nil
		db.LastMod[oldList] = nil
		db.Snapshot[oldList] = nil
		db.SnapshotLM[oldList] = nil

		local payload = { oldList, newName }
		SKG_AddLogEntry( TID, "SKG", "LSTRN", payload, 0 )
	end
end

SKG.OnCommReceive.LSTCL = function ( self, prefix, distribtion, sender, message )
	-- LSTCL
	-- Payload: { TID, OldList, NewName }
	-- Description: Clones list "OldList" with "NewName"

	-- TODO : rewrite security

	local db = SKG.db.profile
	local TID, oldList, newName = unpack( message, 1, 3 )
	if ( db.RollLists[newName] ) then
		-- we've already got one
	else
		SKG_Debug( "Comm: Clone existing list" )

		db.RollLists[newName] = {}
		for key, val in pairs( db.RollLists[oldList] ) do
			db.RollLists[newName][key] = val
		end
		-- duplicate the "LastMod" table, but reset all values
		-- to the current timestamp
		db.LastMod[newName] = {}
		local dateStamp = strsub( TID, 1, 14 )
		for k, v in pairs( db.LastMod[oldList] ) do
			db.LastMod[newName][k] = dateStamp
		end

		SKG_UpdateLists()
		SKG_ResetCurrentList()

		-- if there was any log there leftover from something else, get
		-- initialize it
		db.Log[newName] = {}
		db.Snapshot[newName] = {}
		db.SnapshotLM[newName] = {}

		local payload = { oldList, newName }
		SKG_AddLogEntry( TID, "SKG", "LSTCL", payload, 0 )
		-- Init the new list's Log to a LSTCL event
		db.Log[newName] = {}
		local lHash, sHash = SKG_HashRollList( TID, newName )
		SKG_AddLogEntry( TID, newName, "LSTCL", 0, lHash )
	end
end

------------------------------------------------
-- Player Commands 
------------------------------------------------

function SKG.OnCommReceive:PLRAD( prefix, distribution, sender, message)
	-- PLRAD
	-- Payload: { TID, Player, Class, [PID] }
	-- Description: Adds "Player" to the list of valid "Players"
	--		If "PID" is present, "Player" is an alt of the player with id "PID"

	local PID = ""
	local TID, Player, Class = unpack( message, 1, 3 )
	if ( message[4] ) then
		PID  = message[4]
	end

	SKG_Debug( "class is " .. Class )

	if ( PID == "" ) then
		-- We're inserting a main...
		local dblist = SKG.db.profile.Players

		-- create the player hash
		local PID = SKG_HashName( Player )
		local t = {
			name = Player,
			class = Class
		}
		dblist[PID] = t

		if not ( SKG_SECURITY_LEVEL == "U" ) then
			local payload = { Player, Class }
			SKG_AddLogEntry( TID, "SKG", "PLRAD", payload, 0 )
		end
		SKG_UpdateMainsChooser()
	else	
		-- We're inserting an alt
		local dblist = SKG.db.profile.Alts

		if not dblist[PID] then dblist[PID] = {} end
		local t = {
			name = Player,
			class = Class
		}
		table.insert( dblist[PID], t )

		if not ( SKG_SECURITY_LEVEL == "U" ) then
			local payload = { Player, Class, PID }
			SKG_AddLogEntry( TID, "SKG", "PLRAD", payload, 0 )
		end
	end
	SKG_UpdateSPlayers()
	SKG_UpdateToonChooser()
	SKG_UpdateCurrentList()
	SKG.isAlt = false
	SKG.selectedToon = ""
	SKG.mainPID = ""
	SKG.selectedClass = ""
	SKG.formSubmit = false

	if ( SKG.inRaid == true ) then
		SKG_UpdateRaiders()
	end
end

function SKG.OnCommReceive:PLRIN ( prefix, distribtion, sender, message )
	-- PLRIN
	-- Payload: { TID, List, PID, [Position] }
	-- Description: Adds player with id "PID" into "List"
	-- Details: If "Position" is provided, the player will be added at that
	--		position, otherwise, they will be added to the bottom of the list.

	local Position = ""
	local db = SKG.db.profile
	local TID, listName, PID = unpack( message, 1, 3 )
	if ( message[4] ) then
		Position = message[4]
	end

	-- If we don't have this list, do nothing
	if ( db.RollLists[listName] ) then
		SKG_Debug( "pos = " .. Position )

		local lastMod = strsub( TID, 1, 14 )
		local dblist = SKG.db.profile.RollLists[listName]

		tinsert( dblist, PID )
		db.LastMod[listName][PID] = lastMod
		local Payload = {}

		if ( Position == "" ) then
			db.LastMod[listName][PID] = lastMod
			Payload = { PID }
		else 
			Position = tonumber( Position )
			for i = #dblist, ( Position + 1 ), -1 do
				dblist[i] = dblist[i-1]
			end
			dblist[Position] = PID
			Payload = { PID, Position }
		end
		if not ( SKG_SECURITY_LEVEL == "U" ) then
			local lHash, sHash = SKG_HashRollList( TID, listName )
			SKG_AddLogEntry( TID, listName, "PLRIN", Payload, lHash )
		end

		SKG_UpdateSPlayers()
		SKG_UpdateCurrentList()
		SKG.isAlt = false
		SKG.selectedToon = ""
		SKG.mainPID = ""
		SKG.selectedClass = ""
		SKG.formSubmit = false

		if ( SKG.inRaid == true ) then
			SKG_UpdateRaiders()
		end
	end
end

function SKG.OnCommReceive:PLRDL( prefix, distribution, sender, message )
	-- PLRDL
	-- Payload: { TID, PID, [Player] }
	-- Description:
	--		If no [Player] is provided, this takes the very broad stroke of
	--		removing the player represented by "PID" from all lists ("Players",
	--		"Alts" as well as any RollLists
	--
	--		If the optional [Player] is provided, this will remove the association
	--		from the "Alts" list for [Player] with "PID" (but not remove "PID" from
	--		any lists ("Players" or any RollLists).

	local Player = ""
	local TID, PID = unpack( message, 1, 2 )
	if ( message[3] ) then
		Player = message[3]
	end

	db = SKG.db.profile
	if ( Player == "" ) then
		-- Remove "PID" from all lists.
		-- alts...
		if ( db.Alts[PID] ~= nil ) then
			db.Alts[PID] = nil
		end
		-- loop through roll lists...
		for listName, listData in pairs( db.RollLists ) do
			local lIndex = 1
			local notFound = true
			while ( ( lIndex <= #db.RollLists[listName] ) and ( notFound == true ) ) do
				if ( db.RollLists[listName][lIndex] == PID ) then
					tremove( db.RollLists[listName], lIndex )
					notFound = false
					db.LastMod[listName][PID] = nil
					if not ( SKG_SECURITY_LEVEL == "U" ) then
						local lHash, sHash = SKG_HashRollList( TID, listName )
						SKG_AddLogEntry( TID, listName, "PLRDL", message, lHash )
					end
				end
				lIndex = lIndex + 1
			end
		end
		-- and Players list
		db.Players[PID] = nil
		if not ( SKG_SECURITY_LEVEL == "U" ) then
			SKG_AddLogEntry( TID, "SKG", "PLRDL", message, 0 )
		end
	else
		-- Only remove the alt [Player] from the "Alts" list
		local index = 1
		local notFound = true
		while ( ( index <= #db.Alts[PID] ) and ( notFound == true ) ) do
			if ( db.Alts[PID][index].name == Player ) then
				tremove( db.Alts[PID], index )
				notFound = false
			end
			index = index + 1
		end
		if ( next( db.Alts[PID] ) == nil ) then
			db.Alts[PID] = nil
		end
		if ( notFound == false ) then
			if not ( SKG_SECURITY_LEVEL == "U" ) then
				SKG_AddLogEntry( TID, "SKG", "PLRDL", message, 0 )
			end
		end
	end
	SKG_ScrollFrame.selected = 0
	SKG_UpdateSPlayers()
	SKG_UpdateMainsChooser()
	SKG_UpdateToonChooser()
	SKG_UpdateCurrentList()
end

function SKG.OnCommReceive:PLRRM( prefix, distribution, sender, message )
	-- PLRRM	
	-- Payload: { TID, List, PID }
	-- Description:
	--		Removes player with id "PID" from "List"

	local TID, listName, PID = unpack( message, 1, 3 )
	local db = SKG.db.profile

	-- if we don't have that list, do nothing
	if ( db.RollLists[listName] ) then
		local index = 1
		local notFound = true
		while ( ( index <= #db.RollLists[listName] ) and ( notFound == true ) ) do
			if ( db.RollLists[listName][index] == PID ) then
				tremove( db.RollLists[listName], index )
				db.LastMod[listName][PID] = nil
				notFound = false
			end
			index = index + 1
		end

		if ( notFound == false ) then
			-- this should *always* fire
			if not ( SKG_SECURITY_LEVEL == "U" ) then
				local lHash, sHash = SKG_HashRollList( TID, listName )
				SKG_AddLogEntry( TID, listName, "PLRRM", PID, lHash )
			end
		end
	
		SKG_ScrollFrame.selected = 0
		SKG_UpdateSPlayers()
		SKG_UpdateMainsChooser()
		SKG_UpdateToonChooser()
		SKG_UpdateCurrentList()
	end
end

function SKG.OnCommReceive:PLRMV( prefix, distribution, sender, message )
	-- PLRMV
	-- Payload: { TID, List, PID, Direction }
	-- Description: Moves player identified by PID in"Direction" in "List"
	-- Details: "Direction" will be either "UP" or "DOWN"

	local TID, listName, PID, Direction = unpack( message, 1, 4 )

	local root = SKG.db.profile

	-- if we don't have this list, do nothing
	if ( root.RollLists[listName] ) then
		local db = root.RollLists[listName]

		-- figure out "Position"
		local Position = 0
		for	i = 1, #db do
			if ( db[i] == PID ) then
				Position = i
			end
		end

		if ( Position == 0 ) then
			-- This should never happen, but check anyway
		else
			if ( Direction == "UP") then
				db[Position], db[Position - 1] = db[Position - 1], db[Position]
			else
				db[Position], db[Position + 1] = db[Position + 1], db[Position]
			end

			-- We want to update if the displayed list is the one being modified...
			if ( listName == SKG_ActiveListTitle:GetText() ) then
				-- We're displaying the list being modified.

				-- If the currently selected player was one of the ones that was
				-- moved, the selection should 'follow' the move.
				local SKGs = tonumber( SKG_ScrollFrame.selected )
				if ( Direction == "UP" ) then
					if ( SKGs == Position ) then SKG_ScrollFrame.selected = Position - 1 end
					if ( SKGs == ( Position - 1 )) then SKG_ScrollFrame.selected = Position end
				else
					if ( SKGs == Position ) then SKG_ScrollFrame.selected = Position + 1 end
					if ( SKGs == ( Position + 1 )) then SKG_ScrollFrame.selected = Position end
				end
				SKG_Debug( "select = " .. SKG_ScrollFrame.selected )
				SKG_UpdateCurrentList()
			else
				-- We're displaying something else. Don't change a thing
			end

			if not ( SKG_SECURITY_LEVEL == "U" ) then
				local payload = { PID, Direction }
				local lHash, sHash = SKG_HashRollList( TID, listName )
				SKG_AddLogEntry( TID, listName, "PLRMV", payload, lHash )
			end
		end
	end
end

function SKG.OnCommReceive:PLRSC( prefix, distribution, sender, message )
	-- PLRSC
	-- Payload : { TID,  listName, Player, { PID, PID, ... } }
	-- Description : Suicides "Player" in listName, using the list of PIDs
	--		as the list of people that are affected by the movement. This
	--		typically be anyone in the raid and/or reserved.
	--
	-- Details : "Player" should also be a PID, and will be the player to
	--		be suicided.

	local TID, listName, Player, PIDList = unpack( message, 1, 4 )

	SKG_Debug( "In PLRSC" )
	SKG_Debug( "TID is " .. TID )
	SKG_Debug( "list is " .. listName )
	SKG_Debug( "Player is " .. Player )
	SKG_Debug( "affectedlist is " .. #PIDList .. " entries " )
	local db = SKG.db.profile
	-- don't do anything unless we actually have this list
	if ( db.RollLists[listName] ) then
		-- First, lets create a list of the positions in "ListName" that are
		-- going to move, in the order they appear in the list.
		rollList = db.RollLists[listName]
		local moveList = {}
		local mlIndex = 1
		local foundFirst = false
		for i = 1, #rollList do
			local j = 1
			local notfound = true
			while ( j <= #PIDList ) do
				if ( rollList[i] == PIDList[j] ) then
					notfound = false
					break
				end
				j = j + 1
			end
			if ( notfound == false ) then
				if ( rollList[i] == Player ) then
					foundFirst = true
				end
				-- Only add the person to the moveList if we've already found
				-- the player to be suicided (including that player as well)
				if ( foundFirst == true ) then
					moveList[mlIndex] = i
					mlIndex = mlIndex + 1
				end
			end
		end

		-- At this point, moveList should contain, in order, the positions to move
		SKG_Debug( "our movelist contains " .. #moveList .. " entries." )

		-- Create the datestamp to use for all the LastMod fields.
		local dateStamp = strsub( TID, 1, 14 )
		local lmList = SKG.db.profile.LastMod[listName]

		local moveLen = #moveList
		local SuicideTemp = rollList[moveList[1]]
		local moveIndex = 1
		while ( moveIndex < moveLen ) do
			SKG_Debug( "position in list = " .. moveList[moveIndex] )
			local movingPID = rollList[moveList[moveIndex + 1]]
			SKG_Debug( "movingPID = " .. movingPID )
			rollList[moveList[moveIndex]] = movingPID
			SKG_Debug( "old lastmod entry is " .. lmList[movingPID] )
			lmList[movingPID] = dateStamp
			SKG_Debug( "new lastmod entry is " .. lmList[movingPID] )
			moveIndex = moveIndex + 1
		end
		rollList[moveList[moveIndex]] = SuicideTemp
		lmList[SuicideTemp] = dateStamp

		if not ( SKG_SECURITY_LEVEL == "U" ) then
			local payload = { Player, PIDList }
			local lHash, sHash = SKG_HashRollList( TID, listName )
			SKG_AddLogEntry( TID, listName, "PLRSC", payload, lHash )
		end

		SKG_UpdateCurrentList()
	end
end

------------------------------------------------
-- Loot Commands 
------------------------------------------------

SKG.OnCommReceive.LOOTC = function ( self, prefix, distribtion, sender, message )
	-- LOOTC
	-- Payload: n/a
	-- Description: Clear Loot List

	SKG.LootItems = { }
	SKG_UpdateLootList()
end

SKG.OnCommReceive.LOOTI = function ( self, prefix, distribtion, sender, message )
	-- LOOTI
	-- Payload: { mobGUID, itemLink }
	-- Description: Add itemLink to the Loot List

	local mobGUID, itemLink = unpack( message, 1, 2)
	SKG_Debug( "loot insert -> " .. itemLink .. " (" .. mobGUID .. ")" )
	local t = {
		mGUID = mobGUID,
		link = itemLink
	}

	tinsert( SKG.LootItems, t )
	SKG_UpdateLootList()

	-- If we have autoopen set, pop open to the bid frame
	if ( SKG.db.profile.Options.General.autoopen ) then
		SKG_OpenToBidFrame()
	end

	-- If we have proper permissions (eventually) open up the bidbuttonframe so
	-- a loot event can be handled.
	SKG_BBF_ClassFilterChooser:Enable()
	SKG_BBF_ArmorFilterChooser:Enable()
	SKG_BBF_OpenBidButton:Enable()
	SKG_BBF_CloseBidButton:Disable()
	SKG_BBF_ForceRetractButton:Disable()
	-- note: this does nothing (visible) if the parent frame is hidden
	SKG_BidButtonFrame:Show()
end

SKG.OnCommReceive.LOOTR = function ( self, prefix, distribtion, sender, message )
	-- LOOTR
	-- Payload: { mobGUID, itemLink }
	-- Description: Remove itemLink that dropped off mobGUID from the Loot List

	SKG_Debug( "Received LOOTR" )
	local mobGUID, itemLink = unpack( message, 1, 2 )
	local found = false
	for i = 1, #SKG.LootItems do
		if ( ( SKG.LootItems[i].link == itemLink ) and
			 ( SKG.LootItems[i].mGUID == mobGUID ) ) then
			found = true
			break
		end
	end

	if ( found == true ) then
		tremove( SKG.LootItems, i )
	end

	-- Disable. If there are still loot items, they will re-enable
	-- in UpdateLootList
	SKG_BBF_OpenBidButton:Disable()
	SKG_BBF_CloseBidButton:Disable()
	SKG_BBF_ArmorFilterChooser:Disable()
	SKG_BBF_ClassFilterChooser:Disable()
	SKG_BidButtonFrame:Hide()
	SKG.selectedBidItem = 0
	
	SKG_UpdateLootList()
end

------------------------------------------------
-- Bidding Commands 
------------------------------------------------

SKG.OnCommReceive.BIDRT = function ( self, prefix, distribtion, sender, message )
	-- BIDRT
	-- Payload: bidder
	-- Description: Retract bidder from the current bid list

	local bidder = message

	if ( SKG.CurrBidList ~= "" ) then
		-- we've got a valid roll list
		local db = SKG.db.profile.RollLists[SKG.CurrBidList]

		if ( SKG.ManagingBidEvent == true ) then
			SKG_Debug( "  I'm the event handler" )
		
			-- loop through current bidders
			local removed = 0
			for k, v in pairs( SKG.Bidders ) do
				if ( v == bidder ) then
					removed = k
					break
				end
			end
		end
	end

	if ( removed ~= 0 ) then
		SKG.Bidders[k] = nil
		SKG_UpdateBidderPanel()

		-- If I was the bidder, re-enable my bid button
		SKG_BF_BidButton:Enable()
		SKG_BF_RetractButton:Disable()

		-- If I'm the event handler, announce...
		if ( SKG.ManagingBidEvent == true ) then
			SendChatMessage( string.format( SKGL["SKG_MSG_RETRACTNOTIFY"] , bidder ), "RAID" )
			-- if we have an empty list, send a different message
			if ( next( SKG.Bidders ) == nil ) then
				SendChatMessage( SKGL["SKG_MSG_NOBIDDERS"], "RAID" )
			else
				-- find our high bidder
				local highBidPos = 0
				local highBidder = ""
				for	k, v in pairs( SKG.Bidders ) do
					if ( highBidPos == 0 ) then
						highBidPos = k
						highBidder = v
					else
						if ( k < highBidPos ) then
							highBidPos = k
							highBidder = v
						end
					end
				end
				-- notify
				SendChatMessage( string.format( SKGL["SKG_MSG_HIGHBIDDER"], highBidder ), "RAID" )
			end
		end
	else
		SendChatMessage( SKGL["SKG_ERROR_CANTRETRACT"], "WHISPER", nil, bidder )
		-- our retractor wasn't in the list. just do nothing.
	end
end

SKG.OnCommReceive.BIDBD = function ( self, prefix, distribtion, sender, message )
	-- BIDBD
	-- Payload: bidder
	-- Description: Incoming bid on whatever the current item is, from "bidder"

	local bidder = message

	SKG_Debug( "Received bid event" )

	-- Handle a bunch of error checking up front
	if ( SKG.ManagingBidEvent ~= true ) then
		-- I'm not the loot event handler. 
		return
	end
	SKG_Debug( "I'm the event handler" )

	local foundInRaid = 0
	if ( #SKG.Raiders == 0 ) then
		-- I'm not in a raid
		return
	else
		-- we're in a raid, so find our man
		for i = 1, #SKG.Raiders do
			if ( SKG.Raiders[i].name == bidder ) then
				-- he's in our raid
				foundInRaid = i
				break
			end
		end
		if ( foundInRaid == 0 ) then
			SendChatMessage( SKGL["SKG_ERROR_NOTINMYRAID"], "WHISPER", nil, bidder )
			return
		end
	end
	SKG_Debug( "We're in a raid, and we found our bidder" )

	if ( SKG.CurrBidList == "" ) then
		-- we don't have a bid list. This really shouldn't even happen. Flag an
		-- error the admin, and return
		SKG_Print( SKGL["SKG_ERROR_NOBIDLIST"] )
		return
	end
	SKG_Debug( "current bid list is " .. SKG.CurrBidList )	

	local db = SKG.db.profile.RollLists[SKG.CurrBidList]
	if ( next( db ) == nil ) then
		-- somehow, our roll list no entries. *cry*
		SKG_Debug( "How does our roll list have no entries?" )
		return
	end

	local inList = 0
	for i = 1, #db do
		if ( db[i] == SKG.Raiders[foundInRaid].mPID ) then
			inList = i
			break
		end
	end
	if ( inList == 0 ) then
		SendChatMessage( SKGL["SKG_ERROR_NOTINBIDLIST"], "WHISPER", nil, bidder )
		SKG_Print( SKGL["SKG_ERROR_INVALIDBIDDER"] .. " " .. bidder )
		return
	end
	SKG_Debug( "We've got a valid bidder in our current roll list" )

	-- Looking good so far. Check our loot filter
	local foundClass = 0
	local bClass, enClass = UnitClass( bidder )
	if ( bClass ) then
		for i = 1, 9 do
			if ( enClass == SKG_enClassNames[i] ) then
				foundClass = i
				break
			end
		end
	else
		-- Somehow, the dude didn't return a valid class. He's either not in our
		-- raid (and I can't find the class, or some other weirdness
		return
	end
	if ( foundClass == 0 ) then
		-- somehow, we couldn't find this guy's class in our classlist. Bad. Ignore
		return
	end
	SKG_Debug( "We've got a valid class back from the bidder" )

	SKG_Debug( SKG.BidFilter )
	SKG_Debug( "foundClass = " .. foundClass )
	if ( strsub( SKG.BidFilter, foundClass, foundClass ) ~= "1" ) then
		-- he did not pass our class filter. let him know and bail
		SendChatMessage( SKGL["SKG_ERROR_BIDFILTERED"], "WHISPER", nil, bidder )
		return
	end
	SKG_Debug( "He's made it through our loot filter" )

	-- Has this guy already bid?
	if ( SKG.Bidders[inList] ) then
		-- yikes, he's already bid. Don't let him bid again
		SendChatMessage( SKGL["SKG_ERROR_ALREADYBID"], "WHISPER", nil, bidder )
		return
	end
	SKG_Debug( "He hasn't bid yet" )

	-- If we've made it this far, we're good. THrow caution to the wind and
	-- process the bid

	local payload = { "ADDBIDDER", inList, bidder }
	SKG:SendCommMessage( "GROUP", "SPECL", payload )
end

SKG.OnCommReceive.BIDCL = function ( self, prefix, distribtion, sender, message )
	-- BIDCL
	-- Payload: { TID, mobGUID, ItemLink }
	-- Description: Close the bidding for "ItemLink"

	local TID, mobGUID, itemLink = unpack( message, 1, 3 )

	-- If I'm the one handling a bid event, take care of the end processing
	if ( SKG.ManagingBidEvent == true ) then
		SKG_Debug( "  I'm the event handler" )
		-- do we have any bidders?
		if ( next( SKG.Bidders) ~= nil ) then
			-- we need to:
			-- find out high bidder
			local highBidPos = 0
			local highBidder = ""
			for k, v in pairs( SKG.Bidders ) do
				if ( highBidPos == 0 ) then
					highBidPos = k
					highBidder = v
				else
					if ( k < highBidPos ) then
						highBidPos = k
						highBidder = v
					end
				end
			end
			-- Announce the winner
			SendChatMessage( string.format( SKGL["SKG_MSG_WINNERANNOUNCE"], itemLink, highBidder, highBidPos ), "RAID" )

			-- send a suicide event

			-- Create TID
			local newTID = SKG_CreateTID( 0, 0, 0 )
			local listName = SKG.CurrBidList
			-- create our list of affected PIDs from SKG.Raiders()
			local affectedList = {}
			local winnerPID = 0
			for i = 1, #SKG.Raiders do
				tinsert( affectedList, SKG.Raiders[i].mPID )
				-- while we're in here, figure out which mPID belongs to the winner
				if ( SKG.Raiders[i].name == highBidder ) then
					winnerPID = SKG.Raiders[i].mPID
				end
			end
			-- Add in any reserved players
			for i = 1, #SKG.ReservedPlayers do
				tinsert( affectedList, SKG.ReservedPlayers[i] )
			end
			-- send off the suicide event
			local Payload = { newTID, listName, winnerPID, affectedList }
			SKG:SendCommMessage( "GUILD", "PLRSC", Payload )
		else
			-- Give the DE message
			SendChatMessage( SKGL["SKG_MSG_NOWINNER"], "RAID" )
		end
	end	

	-- for most people, this will just clear out the bidder list and remove
	-- the item from the biddable items list. Also, disable buttons as appropriate
	SKG.Bidders = {}
	SKG_UpdateBidderPanel()

	-- Find the item in our list, and remove it
	for i = 1, #SKG.LootItems do
		if ( mobGUID == SKG.LootItems[i].mGUID ) then
			if ( SKG_SmartCompareItemlinks(
					SKG.LootItems[i].link, itemLink ) == true ) then
				tremove( SKG.LootItems, i )
				break
			end
		end
	end

	-- Disable. If there are still loot items, they will re-enable
	-- in UpdateLootList
	SKG_BBF_OpenBidButton:Disable()
	SKG_BBF_CloseBidButton:Disable()
	SKG_BBF_ArmorFilterChooser:Disable()
	SKG_BBF_ClassFilterChooser:Disable()
	SKG_BidButtonFrame:Hide()
	SKG.selectedBidItem = 0

	SKG_UpdateLootList()

	SKG_BF_BidButton:Disable()
	SKG_BF_RetractButton:Disable()

	SKG.ManagingBidEvent = false
end

SKG.OnCommReceive.BIDOP = function ( self, prefix, distribtion, sender, message )
	-- BIDOP
	-- Payload: { TID, ItemLink, Filter, BidList }
	-- Description: Open the bidding for "ItemLink", filtered as appropriate by
	--		"Filter". For admin purposes, list being used (BidList) is also passed

	local TID, itemLink, filter, bidList = unpack( message, 1, 4 )
	SKG_Debug( "Received BIDOP event" )

	SKG.CurrBidList = bidList
	-- Reset any old bidders list, if there's something left over
	SKG.Bidders = {}
	SKG_UpdateBidderPanel()

	-- If we pass the loot filter, enable our bid button
	local zClass, enClass = UnitClass( "player" )
	local found = 0
	for i = 1, 9 do
		if ( enClass == SKG_enClassNames[i] ) then
			found = i
			break
		end
	end
	if ( found ~= 0 ) then
		if ( strsub( filter, found, found ) == "1" ) then
			-- we pass, enable the button
			SKG_BF_BidButton:Enable()
		else
			-- we don't pass, make sure its disabled
			SKG_BF_BidButton:Disable()
		end
	end

	-- If we have proper permissions (eventually), enable the appropriate buttons
	SKG_BBF_OpenBidButton:Disable()
	SKG_BBF_CloseBidButton:Enable()
	SKG_BBF_ArmorFilterChooser:Disable()
	SKG_BBF_ClassFilterChooser:Disable()
end

------------------------------------------------
-- Special Commands
------------------------------------------------

SKG.OnCommReceive.SPECL = function ( self, prefix, distribtion, sender, message )

	-- This token is for very specific things that needed to be passed between
	-- addons that didn't fit in anywhere else, or were particular "end results"
	-- of some transactions already processed.
	--
	-- The payload passed to this function can be generalized as:
	--	{ "SPECIALTOKEN", ... }
	--
	-- The specific arguments that follow the special token will vary depending on
	-- the token. For now, I'm going to document them inline here...
	--
	-- { "ADDBIDDER", inList, bidder }
	--		inList		- position in the list to insert
	--		bidder		- the name to insert

	local SpecialToken = message[1]

	if ( SpecialToken == "ADDBIDDER" ) then
		local inList, bidder = unpack( message, 2, 3 )

		SKG.Bidders[inList] = bidder
		SKG_UpdateBidderPanel()

		-- If "bidder" is me, then hide my bid button, show the retract button
		if ( bidder == SKG.me ) then
			SKG_BF_BidButton:Disable()
			SKG_BF_RetractButton:Enable()
		end
			
		-- If we happen to the be loot handler, find our high bidder and announce
		if ( SKG.ManagingBidEvent == true ) then
			local highBidPos = 0
			local highBidder = ""
			for k, v in pairs( SKG.Bidders ) do
				if ( highBidPos == 0 ) then
					highBidPos = k
					highBidder = v
				else
					if ( k < highBidPos ) then
						highBidPos = k
						highBidder = v
					end
				end
			end
	
			SendChatMessage( string.format( SKGL["SKG_MSG_BIDNOTIFY"] , bidder, inList ), "RAID" )
			if ( highBidder == "" ) then
				SendChatMessage( string.format( SKGL["SKG_MSG_NOBIDDERS"], highBidder ), "RAID" )
			else 
				SendChatMessage( string.format( SKGL["SKG_MSG_HIGHBIDDER"], highBidder ), "RAID" )
			end
		end
	end
end