-- Assign variables that require localizations to be loaded already

SKG_ClassNames = { 
	SKGL["SKG_CLASS_DRUID"],			-- 1
	SKGL["SKG_CLASS_HUNTER"],			-- 2
	SKGL["SKG_CLASS_MAGE"],				-- 3
	SKGL["SKG_CLASS_PALADIN"],			-- 4
	SKGL["SKG_CLASS_PRIEST"],			-- 5
	SKGL["SKG_CLASS_ROGUE"],			-- 6
	SKGL["SKG_CLASS_SHAMAN"],			-- 7
	SKGL["SKG_CLASS_WARLOCK"],			-- 8
	SKGL["SKG_CLASS_WARRIOR"]			-- 9
}

SKG_enClassNames = {
	"DRUID",
	"HUNTER",
	"MAGE",
	"PALADIN",
	"PRIEST",
	"ROGUE",
	"SHAMAN",
	"WARLOCK",
	"WARRIOR"
}

-- Seed the random number generator:
math.randomseed(math.random(0,2147483647)+(GetTime()*1000));

-- A few utility functions

function SKG_getVersionComponents()
   -- Return mod version as major, minor
   local major = math.floor( SKG_VERSION / 100.0 );

   return major, ( SKG_VERSION -( major * 100 ) );
end

function SKG_Print( msg )
	if not ( msg == nil ) then
	   if( DEFAULT_CHAT_FRAME ) then
	      DEFAULT_CHAT_FRAME:AddMessage( "SKG: " .. msg, 0.6, 1.0, 1.0 )
	   end
	end
end

function SKG_Debug( msg )
	if not ( msg == nil ) then
		if ( SKG_DEBUG ) then
			SKG_Print( "DEBUG: " .. msg )
		end
	end
end

function SKG_SetSecurityLevel()
	-- Criteria:
	-- If user is a Guild Leader, he gets (A)dmin without further ado
	-- If the (A) has assigned a Proxy, check to see if this person is it, set to (P)roxy
	-- If the Security level of "Rank" has been assigned
	--		Check to see if this person is in guild, and meets the rank requirement to be (S)yncer
	-- If the Security option of "List" has been assigned
	--		Check to see if this person is in the list, and assign them (S)yncer
	-- Otherwise, they're a (U)ser

	-- Force a query of the current guild roster so stuff works:
	GuildRoster()

	-- If we're on a guild leader, set his security level
	if ( IsGuildLeader( SKG.me ) ) then 
		SKG_SECURITY_LEVEL = "A"
		SKG_Debug( "you are the guild master" )
	else
		SKG_SECURITY_LEVEL = "U"
		SKG_Debug( "you are not the guild master" )
	end

	-- For testing purposes, just make anyone "A"
	SKG_SECURITY_LEVEL = "A"
end

function SKG_IsGuildLeader()
	-- Is this player the current guild's leader?
	return IsGuildLeader( SKG.me )
end

function SKG_IsMasterLooter()
	local isRaid = UnitInRaid( "player" )
	if ( isRaid == 1 ) then
		local lootMethod, mlPID, mlRID = GetLootMethod()
		if ( lootMethod == "master" ) then
			local numInRaid = GetNumRaidMember()
			local notfound = true
			local i = 1
			while ( ( i <= numInRaid ) and ( notfound == true ) ) do
				local name = GetRaidRosterInfo(i);
				if ( name == UnitName( "player" ) ) then
					if ( mlRID == i ) then
						return true
					end
					notfound = false
				end
				i = i + 1
			end
		end
	end
	return false
end

function SKG_TooltipDisplay( this, textLabel, useTrans )
	this:SetScript( "OnEnter",
		function ( this )
			GameTooltip_SetDefaultAnchor( GameTooltip, this )
			if ( useTrans == 1 ) then
				GameTooltip:SetText( SKGL[ textLabel ], HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b )
			else
				GameTooltip:SetText( textLabel, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b )
			end
			GameTooltip:Show()
		end
	)
	this:SetScript( "OnLeave",
		function( this )
			GameTooltip:Hide()
		end
	)
end

function SKG_DisableNormalText( textObj )
	textObj:SetTextColor( .4, .4, .4, 1 )
end

function SKG_EnableNormalText( textObj )
	textObj:SetTextColor( 1, .82, 0, 1 )
end

function IsOdd( value )
	if ( floor( value / 2 ) == floor( ( value + 1 ) / 2 ) ) then
		return false
	else
		return true
	end
end

function SKG_UpdateRaiders()
	SKG.Raiders = {}
	local numRaiders = GetNumRaidMembers()
	local plList = SKG.db.profile.Players
	local altList = SKG.db.profile.Alts
	local mainPID = 0
	local isAlt = false
	for i = 1, numRaiders do
		mainPID = 0
		isAlt = false
		local raidName, b, b, b, raidClass = GetRaidRosterInfo(i);

		-- we want to store this person's PID, if he's in our raid list
		local found = false
		raidClass = strupper( raidClass )
		for PID, plInfo in pairs( plList ) do
			if ( plInfo.name == raidName ) then
				mainPID = PID
				isAlt = false
				found = true
				break
			end
		end
		-- if we didn't find him in the main list, is it possibly an Alt of someone?
		if ( found == false ) then
			for PID, altInfo in pairs( altList ) do
				for j = 1, #altInfo do
					if ( altInfo[j].name == raidName ) then
						mainPID = PID
						isAlt = true
						found = true
						break
					end
				end
				if ( found == true ) then
					break
				end
			end
		end

		local t = {
			name = raidName,
			class = raidClass,
			raidID = i,
			mPID = mainPID,
			iAlt = isAlt
		}
		tinsert( SKG.Raiders, t )
	end
end

function SKG_TimeStamp()
	-- return datestamp formatted as YYYYMMDDHHMMSS
	local blort = date("*t")

	local outString = blort.year
	if ( blort.month < 10 ) then
		outString = outString .. "0" .. blort.month
	else
		outString = outString .. blort.month
	end
	if ( blort.day < 10 ) then
		outString = outString .. "0" .. blort.day
	else
		outString = outString .. blort.day
	end
	if ( blort.hour < 10 ) then
		outString = outString .. "0" .. blort.hour
	else
		outString = outString .. blort.hour
	end
	if ( blort.min < 10 ) then
		outString = outString .. "0" .. blort.min
	else
		outString = outString .. blort.min
	end
	if ( blort.sec < 10 ) then
		outString = outString .. "0" .. blort.sec
	else
		outString = outString .. blort.sec
	end

	return outString
end

function SKG_DaysSinceEpoch( inString )
--	SKG_Debug( "SKG_DaysSinceEpoch: " .. inString )
	inYear = string.sub( inString, 7, 10 )
	inYear = inYear - 1
	inMonth = string.sub( inString, 1, 2 )
	inMonth = inMonth - 1
	inDay = string.sub( inString, 4, 5 )
	inDay = inDay - 1 + 1
	return ( inYear * 365 ) + ( inMonth * 30 ) + ( inDay * 1 )
end

function SKG_TimeSince( inString )
	-- takes a string in format YYYYMMDDHHMMSS
	-- and returns the time in days since last activity (eg, "3 days")
	local currentDate = date( "*t" )
--	SKG_Debug( "SKG_TimeSince: " .. inString )
	local inYear = string.sub( inString, 1, 4 )
	local inMonth = string.sub( inString, 5, 6 )
	local inDay = string.sub( inString, 7, 8 )

	local outString = ""
	if ( currentDate.month < 10 ) then
		outString = "0" .. currentDate.month .. "/"
	else
		outString = currentDate.month .. "/"
	end
	if ( currentDate.day < 10 ) then
		outString = outString .. "0" .. currentDate.day .. "/"
	else
		outString = outString .. currentDate.day .. "/"
	end
	outString = outString .. currentDate.year

	return SKG_DaysSinceEpoch( outString ) - SKG_DaysSinceEpoch( inMonth .. "/" .. inDay .. "/" .. inYear ) .. " " .. SKGL["SKG__DAYS"]
end

function SKG_PrettyTimeStamp ( inString )
	-- Takes a timestamp in format YYYYMMDDHHMMSS
	-- and formats it: "MM/DD/YYYY, HH:MM:SS"
	outString = string.sub( inString, 5, 6 ) ..
		"/" .. string.sub( inString, 7, 8 ) ..
		"/" .. string.sub( inString, 1, 4 ) ..
		", " .. string.sub( inString, 9, 10 ) ..
		":" .. string.sub( inString, 11, 12 ) ..
		":" .. string.sub( inString, 13, 14 )

	return outString
end

function SKG_CreateButton(
	bName, parentFrame,	bHeight, bWidth,
	anchorFrom, anchorFrame, anchorTo, xOffset, yOffset,
	bNormFont, bHighFont, initText, tAlign, tooltipText, noTextures )

	-- I hate stretchy buttons. Thanks very much to ckknight for this code
	-- (found in RockConfig)

	-- when pressed, the button should look pressed
	local function button_OnMouseDown(this)
		if this:IsEnabled() == 1 then
			this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Down]])
			this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Down]])
			this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Down]])
		end
	end
	-- when depressed, return to normal
	local function button_OnMouseUp(this)
		if this:IsEnabled() == 1 then
			this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
			this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
			this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		end
	end

	local function button_Disable(this)
		this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Disabled]])
		this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Disabled]])
		this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Disabled]])
		this:__Disable()
		this:EnableMouse(false)
	end

	local function button_Enable(this)
		this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		this:__Enable()
		this:EnableMouse(true)
	end

	local button = CreateFrame( "Button", bName, parentFrame )

	button:SetWidth( bWidth )
	button:SetHeight( bHeight )

	if not ( noTextures == 1 ) then
		local left = button:CreateTexture(button:GetName() .. "_LeftTexture", "BACKGROUND")
		button.left = left
		local middle = button:CreateTexture(button:GetName() .. "_MiddleTexture", "BACKGROUND")
		button.middle = middle
		local right = button:CreateTexture(button:GetName() .. "_RightTexture", "BACKGROUND")
		button.right = right

		left:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		right:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])

		left:SetPoint("TOPLEFT")
		left:SetPoint("BOTTOMLEFT")
		left:SetWidth(12)
		left:SetTexCoord(0, 0.09375, 0, 0.6875)

		right:SetPoint("TOPRIGHT")
		right:SetPoint("BOTTOMRIGHT")
		right:SetWidth(12)
		right:SetTexCoord(0.53125, 0.625, 0, 0.6875)

		middle:SetPoint("TOPLEFT", left, "TOPRIGHT")
		middle:SetPoint("BOTTOMRIGHT", right, "BOTTOMLEFT")
		middle:SetTexCoord(0.09375, 0.53125, 0, 0.6875)

		button:SetScript("OnMouseDown", button_OnMouseDown)
		button:SetScript("OnMouseUp", button_OnMouseUp)
		button:SetScript("OnEnter", SubControl_OnEnter)
		button:SetScript("OnLeave", SubControl_OnLeave)

		button.__Enable = button.Enable
		button.__Disable = button.Disable
		button.Enable = button_Enable
		button.Disable = button_Disable

		local highlight = button:CreateTexture(button:GetName() .. "_Highlight", "OVERLAY", "UIPanelButtonHighlightTexture")
		button:SetHighlightTexture(highlight)
	end

	local text = button:CreateFontString(button:GetName() .. "_FontString", "ARTWORK")
	button:SetFontString(text)
	button.text = text
	text:SetPoint("LEFT", button, "LEFT", 7, 0)
	text:SetPoint("RIGHT", button, "RIGHT", -7, 0)
	text:SetJustifyH( tAlign )

	button:SetTextFontObject(bNormFont)
	button:SetHighlightFontObject(bHighFont)
	button:SetDisabledFontObject(GameFontDisableSmall)

	text:SetText( initText )		

	button:SetPoint( anchorFrom, anchorFrame, anchorTo, xOffset, yOffset )
	if not ( tooltipText == "" ) then
		SKG_TooltipDisplay( button, tooltipText, 1 )
	end

	return button
end

function SKG_AddLogEntry( TID, List, CommandToken, Payload, newlistHash )
	if ( type( Payload ) == "table" ) then
		SKG_Debug( "ALE : TID " .. TID .. ", List " .. List ..
			", Token " .. CommandToken .. ", PL is table" )
	else
		SKG_Debug( "ALE : TID " .. TID .. ", List " .. List ..
			", Token " .. CommandToken .. ", PL " .. Payload )
	end

	local db = SKG.db.profile.Log
	-- If we're not a syncer, don't log
	if ( SKG_SECURITY_LEVEL == "U" ) then
		-- Don't do anything
	else
		local addEntry = 0
		if ( List == "INIT" ) then
			-- This is our first entry. Just add it
			List = "SKG"
			db[List] = {}
			addEntry = 1
		else
			-- Does a log entry exist already for the current TID?
			if ( not db[List][TID] ) then
				addEntry = 1
			else
				-- We've already got one, you see. Go away or I shall taunt
				-- you a second time
				SKG_Debug( "log entry already existed" )
			end
		end
		if ( addEntry == 1 ) then
			SKG_Debug( "addentry was 1, attempting to add" )
			local t = {
				token = CommandToken,
				payload = Payload,
				nlHash = newlistHash
			}
			db[List][TID] = {}
			db[List][TID] = t
		end
	end
end

function SKG_CreateTID( index, itemid, uniqueid )
	local outString = SKG_TimeStamp()
	outString = outString .. index
	-- pad the itemid to make it 8 characters
	outString = outString .. strrep( "0", ( 8 - strlen( itemid ) ) ) .. itemid
	-- documentation shows that uniqueid can be negative. make it positive
	outString = outString .. abs( uniqueid )
	return outString
end

function SKG_HashName( name )
	-- Since usernames are, at maximum 12 unicode characters, my output should 2 x 12 = 24 hex characters
	-- Note, for unicode special characters (eg, \195\167), the \195 part will be ignored


	local outString = ""
	for i = 1, strlen( name ) do
		local bv = strbyte( name, i )
		if ( bv == 195 ) then
			-- ignore. I don't want the special codes. just the characters
		else
			outString = outString .. string.format( "%x", bv )
		end
	end
	while ( strlen( outString ) < 24 ) do
		outString = outString .. "32"
	end	
	return strupper( outString )
end

function SKG_HashRollList( TID, rollList )
	-- Method:
	--	Step 1: 
	--		convert each letter in a name to a number (1-255)
	--		add them together (max value : 12 * 255 = 3060
	--		add the list position 3060 + 450 = 3510
	--	Step 2:
	--		add groups of 9 names together ( 9 * 3510 = 31590)
	--		convert to hex (4 characters)
	--
	-- Non-arbitrary limitation of lists to 450 raiders
	-- This will yield 450/9 = 50 groups x 4 characters = 200 character hash
	-- Timestamp + the above string is the FULL hash of the list, yielding a maximum
	-- of 214 characters
	-- 
	-- For display purposes (sync page), we need to rehash to a smaller string
	--
	--	Step 3:
	--		take groups of 16 hex characters, and add them together
	--		highest possible value = 16 x 16 = 256
	--		divide by 2, highest value = 128 (2 hex characters)
	--		177 / 16 = 12 sets * 2 characters = 24 characters
	--	Step 4:
	--		add in the Timestamp digits, don't carry ones
	--
	-- Thus, the shorthash will ALWAYS be 24 "digits"
	--
	-- If the list in question is "Players" then, this simplifies considerably
	-- since I don't care about the TID or list position. Just add everything
	-- together. The format for returned data is:
	--		longhash = PlayerHash (#Players), shorthash = AltHash (#Alts)
	--

	local outString = ""
	local tempValue = 0
	local altValue = 0
	local newHash = ""
	if ( rollList == "Players" ) then
		local numPlayers = 0
		for nameHash, playerTable in pairs( SKG.db.profile.Players ) do
			numPlayers = numPlayers + 1
			local tName = playerTable.name
			for k = 1, strlen( tName ) do
				tempValue = tempValue + strbyte( tName, k )
			end
		end
		-- convert to hex
		outString = outString .. string.format( "%x", tempValue )
		if ( string.len( outString ) > 8 ) then
			outString = string.sub( outString, 1, 8 )
		end
		outString = outString .. "(" .. numPlayers .. ")"

		local altPlayers = 0
		dbAlts = SKG.db.profile.Alts
		for hash, altList in pairs( dbAlts ) do
			for altIndex, plInfo in pairs( dbAlts[hash] ) do
				altPlayers = altPlayers + 1
				local tName = altList[altIndex].name
				for k = 1, strlen( tName ) do
					altValue = altValue + strbyte( tName, k )
				end
			end
		end
		newHash = newHash .. string.format( "%x", altValue )
		if ( string.len( newHash ) > 8 ) then
			newHash = string.sub( newHash, 1, 8 )
		end
		newHash = newHash .. "(" .. altPlayers .. ")"
	else
		outString, newHash = SKG_GenericListHash( TID, SKG.db.profile.RollLists[rollList] )
	end

	return strupper( outString ), strupper( newHash )
end

function SKG_InitAllPanels()
	SKG_Debug( "In InitAllPanels" )
	-- We've switched profiles, and have new settings. Make sure all options
	-- and checkboxes are appropriately set
	local db = SKG.db.profile

	-- If we've switched to an empty profile, at least start with the defaults
	-- This code shouldn't be necessary. We should automatically get defaults...
	if not db.firstLoad then
		SKG_Debug( "Setting defaults" )
		db.firstLoad = true
		db.Players = {}
		db.Alts = {}
		db.RollLists = {}
		db.LastMod = {}
		db.Options = {}
		db.Log = {}
		db.Log["SKG"] = {}
		db.Snapshot = {}
		db.SnapshotLM = {}
		db.Options.General = {}
		db.Options.General = {
			debug = false,
			autoopen = false,
			buttonTexture = 1,
			classColors = true,
			raidonly = false,
		}
		db.Options.Security = {}
		db.Options.Bid = {}
		db.Options.List = {}
	end

	-- Start with Options panel
	local dbGO = db.Options.General
	dbGO.xPos = SKG_Frame:GetLeft()
	dbGO.yPos = SKG_Frame:GetTop()
	SKG_OF_RaidOnlyCB:SetChecked( dbGO.raidonly )
	SKG_OF_ShowAllCB:SetChecked( not dbGO.raidonly )
	SKG_OF_ShowClassColorsCB:SetChecked( dbGO.classColors )
	SKG_SetOptionSwitcherButtonTexture( dbGO.buttonTexture )
	local profile = SKG:GetProfile()
	if ( profile == "Default" ) then
		SKG_OF_ProfileCB:SetChecked( true )
		SKG_OF_ProfileCopy:Disable()
	else
		SKG_OF_ProfileCB:SetChecked( false )
		SKG_OF_ProfileCopy:Enable()
	end
	SKG_OF_AutoOpenCB:SetChecked( dbGO.autoopen )
	SKG_DEBUG = dbGO.debug
	SKG_OF_DebugCB:SetChecked( dbGO.debug )

	-- Lists:
	-- Initialize the dropdown list with the current saved lists...
	SKG_UpdateLists()

	local dbLO = SKG.db.profile.Options.List
	if not ( dbLO.AutoAdd ) then
		dbLO.AutoAdd = false
		SKG_LF_AutoAddCB:SetChecked( false )
	else
		dbLO.AutoAdd = true
		SKG_LF_AutoAddCB:SetChecked( true )
	end

	-- Bid stuff:
	if ( db.Options.Bid.ArmorFilter == 0 ) then
		SKG_BBF_ArmorFilterText:SetText( SKGL["SKG_LBL_BBF_ARMORFILTER_TEXT_NONEFULL"] )
	elseif ( db.Options.Bid.ArmorFilter == 1 ) then
		SKG_BBF_ArmorFilterText:SetText( SKGL["SKG_LBL_BBF_ARMORFILTER_TEXT_LOOSEFULL"] )
	else
		SKG_BBF_ArmorFilterText:SetText( SKGL["SKG_LBL_BBF_ARMORFILTER_TEXT_STRICTFULL"] )
	end

	SKG_UpdateLootList()

	-- check if we're in a raid.
	local numRaiders = GetNumRaidMembers()
	if ( numRaiders > 0 ) then
		SKG.inRaid = true
		SKG_UpdateRaiders()

		-- if we also happen to be raid leader, then enable this stuff too
		if ( IsRaidLeader() == 1 ) then
			SKG_BF_LootTypeButton:Show()
			SKG_BF_LootThreshButton:Show()
			SKG_BF_LootTypeButton:Enable()
			SKG_BF_LootThreshButton:Enable()

			-- Build the initial list of people in the ML list.
			-- scan through the SKG.Raiders struct, and put entries in the list
			SKG.lootTypeSelections.args[3].args = {}
			for i = 1, #SKG.Raiders do
				raidName = SKG.Raiders[i].name
				raidIndex = SKG.Raiders[i].raidID
				SKG_Debug( "n = " .. raidName .. ", rI = " .. raidIndex )
				local a = {
					type = "execute",
					name = raidName,
					desc = raidName,
					func = function( value )
						-- Set Master Looter to this player
						SetLootMethod( "master", value )
						Dewdrop:Close()
					end,
					passValue = raidName,
					order = raidIndex,
				}
				table.insert( SKG.lootTypeSelections.args[3].args, a )
			end
		else
			SKG_BF_LootTypeButton:Hide()
			SKG_BF_LootThreshButton:Hide()
		end
	else
		SKG_BF_LootTypeButton:Hide()
		SKG_BF_LootThreshButton:Hide()
	end
	-- Disabled until looting events occur
	SKG_UpdateBidderPanel()
	SKG_BF_BidButton:Disable()
	SKG_BF_RetractButton:Disable()
	SKG_BidButtonFrame:Hide()

	-- Text for the title...
	local major, minor = SKG_getVersionComponents()
	SKG_TitleText:SetText( string.format( SKGL["SKG_LBL_TITLE_TEXT"] , 
						major, minor, SKG_SECURITY_LEVEL) )

	-- Until I've received a SECUR packet, and "Security" is something
	-- other than nil, pretty much everything gets disabled.
	if ( next( db.Options.Security ) == nil ) then
		SKG_LF_NewListButton:Disable()
		SKG_LF_DelListButton:Disable()
		SKG_LF_RenListButton:Disable()
		SKG_LF_CloneListButton:Disable()
		SKG_LF_SnapshotListButton:Disable()
		SKG_LF_MoveUpButton:Disable()
		SKG_LF_MoveDownButton:Disable()
		SKG_LF_InsertPlayerButton:Disable()
		SKG_LF_RemovePlayerButton:Disable()
		SKG_LF_SuicidePlayerButton:Disable()
		SKG_LF_ReservePlayerButton:Disable()
		SKG_LF_FindMissingButton:Disable()
		SKG_LF_AddMissingButton:Disable()
	else
		-- Security stuff
		-- this all needs a complete rewrite
	end

	-- Since I'm currently working with security disabled completely, just re-enable
	-- everything I just disabled above. Eventually, this will actually behave
	-- according to the security

	-- This has been disabled for now...
	SKG_LF_NewListButton:Enable()
	SKG_LF_DelListButton:Enable()
	SKG_LF_RenListButton:Enable()
	SKG_LF_CloneListButton:Enable()
	SKG_LF_SnapshotListButton:Enable()
	SKG_LF_MoveUpButton:Enable()
	SKG_LF_MoveDownButton:Enable()
	SKG_LF_InsertPlayerButton:Enable()
	SKG_LF_RemovePlayerButton:Enable()
	SKG_LF_SuicidePlayerButton:Enable()
	SKG_LF_ReservePlayerButton:Enable()
	SKG_LF_FindMissingButton:Enable()
	SKG_LF_AddMissingButton:Enable()

	-- Display lists now...
	SKG_UpdateLists()
end

function SKG_SetLootFilter ( bitMap )
	SKG.LootFilter = bitMap
	for index = 1, 9 do
		local tempT = SKG_BFF_ClassFilterText[index]
		if ( string.sub( bitMap, index, index ) == "1" ) then
			tempT:SetText( "[ X ] " .. SKG_ClassNames[index] )
		else
			tempT:SetText( "[ _ ] " .. SKG_ClassNames[index] )
		end
	end
	SKG_Debug( "LF = " .. SKG.LootFilter )
end

function SKG_SetSingleLootFilter ( bitMap )
	local index = strfind( bitMap, "1" )

	local beginS, endS, currV = ""
	beginS = strsub( SKG.LootFilter, 0, (index - 1) )
	endS = strsub( SKG.LootFilter, (index + 1) )
	currV = strsub( SKG.LootFilter, index, index )
	local tempT = SKG_BFF_ClassFilterText[index]
	if ( currV == "1" ) then
		SKG.LootFilter = beginS .. "0" .. endS
		tempT:SetText( "[ _ ] " .. SKG_ClassNames[index] )
	else
		SKG.LootFilter = beginS .. "1" .. endS
		tempT:SetText( "[ X ] " .. SKG_ClassNames[index] )
	end
	SKG_Debug( "LF = " .. SKG.LootFilter )
end

function SKG_CombineLootFilters ( )
	SKG_Debug( "LF = " .. SKG.LootFilter )
	SKG_UpdateLootFilters()
	local retFilter = ""
	for i = 1, 9 do
		local mf = strsub( SKG.LootFilter, i, i )
		local af = strsub( SKG.afModify, i, i )
		if ( ( mf == "1" ) and ( af == "1" ) ) then
			retFilter = retFilter .. "1"
		else
			retFilter = retFilter .. "0"
		end
	end
	return retFilter
end

function SKG_UpdateLootFilters ( ) 
	-- if we have an item selected, go ahead and get details, update
	-- SKG.afModify based on that.
	if ( SKG.selectedBidItem ~= 0 ) then
		local itemLink = SKG_BF_BidItems[SKG.selectedBidItem]:GetText()

		local itemID = string.match( itemLink,
			"|%w+|Hitem:(%d+):%d+:%d+:%d+:%d+:%d+:%-?%d+:%-?%d+|h%[.-%]|h|r" )
		local _, _, _, _, _, itemType, itemSubType = GetItemInfo( itemLink )

		if ( SKG_LootFilter[itemID] ) then
			local thisLF = SKG_LootFilter[itemID]
			SKG_SetLootFilter( thisLF )
			SKG_Print( SKGL["SKG_MSG_LFSET"] )
		end

		-- Change our armor modifier, if appopriate
		if ( itemType == "Armor") then
			if ( ( itemSubType == "Cloth" ) or
				 ( itemSubType == "Leather" ) or
				 ( itemSubType == "Mail" ) or
				 ( itemSubType == "Plate" ) ) then
				local af = SKG.db.profile.Options.Bid.ArmorFilter
				SKG.afModify = SKG_ArmorFilter[af][itemSubType]
			else
				SKG.afModify = "111111111"
			end
		else
			SKG.afModify = "111111111"
		end
	end
end

function SKG_ClassifyLootFilter( filter )
	local retStr = ""
	for i = 1, 9 do
		local f = strsub( filter, i, i )
		if ( f == "1" ) then
			if ( retStr == "" ) then
				retStr = retStr .. SKG_ClassNames[i]
			else
				retStr = retStr .. " " .. SKG_ClassNames[i]
			end
			SKG_Debug( retStr )
		end
	end

	return retStr
end

function SKG_GetMyRaidID( )
	local unitName = ""
	for index = 1, 40 do
		unitName = GetRaidRosterInfo( index )
		if not ( unitName == nil ) then
			if ( unitName == SKG.me ) then
				return index
			end
		end
	end
	return 0
end

function SKG_GetOldestTID( logList )
	local minTID = 0
	local minTIDs = ""

	if ( logList == nil ) then
		return 0
	end
	
	for index, tval in pairs( logList ) do
		local conv = tonumber(index)
		if ( minTID == 0 ) then
			minTID = conv
			minTIDs = index
		else
			if ( conv < minTID ) then
				minTID = conv
				minTIDs = index
			end
		end
	end

	return minTIDs
end

function SKG_GetLatestTID( logList )
	local maxTID = 0
	local maxTIDs = ""

	if ( logList == nil ) then
		return 0
	end

	for index, tval in pairs( logList ) do
		local conv = tonumber( index )
		if ( conv > maxTID ) then
			maxTIDs = index
			maxTID = conv
		end
	end

	return maxTIDs
end

function SKG_AddAlphaGroups( table )
	for alphaIndex = 1, 5 do
		if ( alphaIndex == 1 ) then
			nameDesc = "A-C"
		elseif ( alphaIndex == 2 ) then
			nameDesc = "D-F"
		elseif ( alphaIndex == 3 ) then
			nameDesc = "G-L"
		elseif ( alphaIndex == 4 ) then
			nameDesc = "M-R"
		else
			nameDesc = "S-Z"
		end
		local v = {
			type = "group",
			name = nameDesc,
			desc = nameDesc,
			order = 100 + alphaIndex,
			args = { },
		}
		tinsert( table.args, v )
	end
end

function SKG_UpdateMainsChooser ( )
	SKG.mainSelections.args = {}
	SKG_AddAlphaGroups( SKG.mainSelections )
	SKG.mainPID = ""
	local dblist = SKG.db.profile.Players
	SKG_LFIPF_AltCB:SetChecked( false )

	SKG_Debug( "in UpdateMainsChooser" )
	if ( next( dblist ) == nil ) then
		-- we don't have any mains yet. disable the "Alt" selector
		SKG_LFIPF_MainChooser:Disable()
		SKG_LFIPF_AltCB:Disable()
		SKG_DisableNormalText( SKG_LFIPF_AltCBText )
	else 
		-- We've got entries, fill 'em in.
		SKG_LFIPF_MainChooser:Enable()
		SKG_LFIPF_AltCB:Enable()
		SKG_EnableNormalText( SKG_LFIPF_AltCBText )
		for PID,playerInfo in pairs( dblist ) do
			local pValue = PID .. "|" .. dblist[PID].name
			local t = {
				type = "execute",
				name = dblist[PID].name,
				desc = dblist[PID].name,
				func = function ( crapola )
					pID, pName = strsplit( "|", crapola )
					SKG.mainPID = pID
					SKG_LFIPF_MainChooser:SetText( SKGL["SKG_LBL_LFIPF_SELECTMAIN_TEXT"] .. ": " .. pName )
					Dewdrop:Close()
				end,
				passValue = pValue
			}
			local AI = alphaIndex( dblist[PID].name )
			tinsert( SKG.mainSelections.args[AI].args, t )
		end
	end
end

function SKG_UpdateToonChooser( )
	SKG.toonSelections.args = {}
	SKG_AddAlphaGroups( SKG.toonSelections )

	SKG.selectedToon = ""
	SKG.mainPID = ""
	local dblist = SKG.db.profile.Players
	local altlist = SKG.db.profile.Alts

	SKG_Debug( "in UpdateToonChooser" )
	if ( next( dblist) == nil ) then
		-- we don't have any mains yet. disable the "Alt" selector
		SKG_LFTCF_ToonChooser:Disable()
	else 
		-- We've got entries, fill 'em in.
		SKG_LFTCF_ToonChooser:Enable()
		local index = 1
		for PID,playerInfo in pairs( dblist ) do
			local mOrder = index * 100
			local pValue = PID .. "|" .. dblist[PID].name
			local t = {
				type = "execute",
				name = dblist[PID].name,
				desc = dblist[PID].name,
				func = function ( crapola )
					pID, pName = strsplit( "|", crapola )
					SKG.selectedToon = pName
					SKG.mainPID = pID
					SKG.isAlt = false
					SKG_LFTCF_ToonChooser:SetText( SKGL["SKG_LBL_LFTCF_SELECTTOON_TEXT"] .. ": " .. pName )
					Dewdrop:Close()
				end,
				order = mOrder,
				passValue = pValue
			}
			local AI = alphaIndex( dblist[PID].name )
			tinsert( SKG.toonSelections.args[AI].args, t )

			-- If this toon has any alts, list them here...
			if ( altlist[PID] ) then
				for i2 = 1, #altlist[PID] do
					local aOrder = mOrder + i2
					local pValue = PID .. "|" .. altlist[PID][i2].name
					local alt = {
						type = "execute",
						name = "- " .. altlist[PID][i2].name,
						desc = "Alt of " .. dblist[PID].name,
						func = function ( crapola )
							pID, pName = strsplit( "|", crapola )
							SKG.selectedToon = pName
							SKG.mainPID = pID
							SKG.isAlt = true
							SKG_LFTCF_ToonChooser:SetText( SKGL["SKG_LBL_LFTCF_SELECTTOON_TEXT"] .. ": " .. pName )
							Dewdrop:Close()
						end,
						order = aOrder,
						passValue = pValue
					}
					tinsert( SKG.toonSelections.args[AI].args, alt )
				end
			end
			index = index + 1
		end
	end
end

function alphaIndex( blort )
	local AI = 0
	firstLetter = string.sub( blort, 1, 1 )
	if ( firstLetter == "A" or
		firstLetter == "B" or
		firstLetter == "C" ) then AI = 1
	elseif ( firstLetter == "D" or
		firstLetter == "E" or
		firstLetter == "F" ) then AI = 2
	elseif ( firstLetter == "G" or 
		firstLetter == "H" or
		firstLetter == "I" or
		firstLetter == "J" or
		firstLetter == "K" or
		firstLetter == "L" ) then AI = 3
	elseif ( firstLetter == "M" or
		firstLetter == "N" or
		firstLetter == "O" or
		firstLetter == "P" or
		firstLetter == "Q" or
		firstLetter == "R" ) then AI = 4
	else AI = 5 end
	return AI
end

--[[ This probably isn't needed now since I'm gonna use SPlayers
function tlen( t )
	-- need a function to get the length of non-numeric indexed tables.
	-- This makes me cry. 
	local i = 0
	for x,y in pairs( t ) do
		i = i + 1
	end
	return i
end
]]--

function SKG_UpdateSPlayers ( )
	SKG.SPlayers = {}
	for PID, pInfo in pairs( SKG.db.profile.Players ) do
		tinsert( SKG.SPlayers, PID )
	end
	table.sort( SKG.SPlayers, function ( x, y )
		if ( SKG.db.profile.Players[x].name <= SKG.db.profile.Players[y].name ) then
			return true
		else
			return false
		end
	end )
end

function SKG_GenericListHash ( TID, gList )
	local tempValue = 0
	local outString = ""
	local newHash = ""

	if ( next( gList ) == nil ) then
		return 0, 0
	end

	local pList = SKG.db.profile.Players
	local timeStamp = strsub( TID, 1, 14 )
	local i, j, k = 0, 0, 0
	j = 1
	for i = 1, #gList do
		local tName = pList[gList[i]].name
		if ( j == 9 ) then
			for k = 1, strlen( tName ) do
				tempValue = tempValue + strbyte( tName, k )
			end
			tempValue = tempValue + i -- adds in list position
			outString = outString .. string.format( "%x", tempValue )
			tempValue = 0
			j = 1
		else
			for k = 1, strlen( tName ) do
				tempValue = tempValue + strbyte( tName, k )
			end
			j = j + 1
		end
	end
	if not ( tempValue == 0 ) then
		outString = outString .. string.format( "%x", tempValue )
	end
	outString = timeStamp .. outString

	-- now for the shorter hash:

	tempValue = 0
	local shortHash = ""
	local moreZeroes = 16 - math.fmod( strlen( outString ), 16 )
	local newString = ""
	if ( moreZeroes < 16 ) then
		newString = strrep( "0", moreZeroes ) .. outString
	else
		newString = outString
	end
	for i = 1, ( strlen( newString ) / 16 ) do
		local portion = strsub( newString, ( 16 * ( i - 1 )) + 1, ( 16 * i ) )
		for j = 1, 16 do
			tempValue = tempValue + ( ( "0x" .. strsub( portion, j, j ) ) + 0 )
		end
		tempValue = floor( tempValue / 2 )
		shortHash = shortHash .. string.format( "%x", tempValue )
		tempValue = 0
	end
	
	-- add in the timestamp
	for index = -24, -1, 1 do
		local tsC = strsub( TID, index, index )
		local shC = strsub( shortHash, index, index )
		if ( shC == "" ) then
			newHash = newHash .. tsC
		else
			local blort = string.sub( string.format( "%X", ( "0x" .. tsC ) + ( "0x" .. shC ) ), -1, -1 )
			newHash = newHash .. blort
		end
	end
	return outString, newHash
end

function SKG_RunTransactions ( transList, targetList, lastmodList )
	-- This function is just a coordinated place to put all the various
	-- handlers for each type of transaction against a roll list.
	-- "transList" is the list of transactions to run, in no particular
	-- order, which "targetList" is the Roll list to run the transactions
	-- against

	-- first, we need to sort transList to be in time order...
	SKG_Debug( "Sorting transaction list..." )
	local sortedTrans = {}
	for k, v in pairs( transList ) do
		if ( #sortedTrans == 0 ) then
			tinsert( sortedTrans, k )
		else
			local i = 1
			local found = false
			while ( ( i <= #sortedTrans ) and ( found == false ) ) do
				if ( k < sortedTrans[i] ) then
					found = true
				else
					i = i + 1
				end
			end
			if ( found == false ) then
				tinsert( sortedTrans, k )
			else
				tinsert( sortedTrans, i, k )
			end
		end
	end
	SKG_Debug( "Sorted entries " .. #sortedTrans )

	-- One of two things must be true here:
	--		1) the first event is SYNSS with a hash of 0 (fresh start to list) and
	--		   our targetList is empty, or
	--		2) the first event is SYNSS with a NON-zero hash, and out targetList's
	--		   hash matches that hash
	
	local currentTID = sortedTrans[1]
	if ( transList[currentTID].token == "SYNSS" ) then
		if ( transList[currentTID].nlHash == 0 ) then
			-- Our targetlist better not have any entries in it yet
			if ( next( targetList ) == nil ) then
				-- good deal. we can proceed
				SKG_Debug( "good case #1" )
			else
				SKG_Debug( "RunTransactions called, SYNSS with 0 hash" )
				SKG_Debug( "but targetlist is not empty" )
				return
			end
		else
			-- we're non-zero. Match the current hash
			local thisHash = SKG_GenericHashList( currentTID, targetList )
			if ( thisHash == transList[currentTID].nlHash ) then
				-- good to go.
				SKG_Debug( "good case #2" )
			else
				SKG_Debug( "RunTransactions called, SYNSS with !0 hash" )
				SKG_Debug( "hash or targetlist != SYNSS hash" )
				return
			end
		end
	else
		-- we've called this routine with bad data. Try again
		SKG_Debug( "RunTransactions called, but first transaction is NOT SYNSS")
		return
	end

	-- loop through remaining transaction
	for transIndex = 2, #sortedTrans do
		SKG_Debug( "Transloop " .. transIndex )
		SKG_Debug( "size of targetList is " .. #targetList )
		local currentTID = sortedTrans[transIndex]
		local currentHash = transList[currentTID].nlHash
		local currentToken = transList[currentTID].token
		local currentPayload = transList[currentTID].payload

		local currentLM = strsub( currentTID, 1, 14 )
		if ( currentToken == "PLRIN" ) then
			-- payload will be { PID } or { PID, Position }
			SKG_Debug( "PLRIN" )
			local PID, Position = "", ""
			if ( #currentPayload == 2 ) then
				PID, Position = unpack( currentPayload, 1, 2 )
			else
				PID = currentPayload[1]
			end

			tinsert( targetList, PID )
			lastmodList[PID] = currentLM
			if ( Position ~= "" ) then
				for i = #targetList, ( Position + 1 ), -1 do
					targetList[i] = targetList[i-1]
				end
				targetList[Position] = PID
			end
		elseif ( currentToken == "PLRRM" ) then
			-- payload will be PID 
			SKG_Debug( "PLRRM" )

			local PID = currentPayload
			local index = 1
			local notFound = true
			while ( ( index <= #targetList ) and ( notFound == true ) ) do
				if ( targetList[index] == PID ) then
					tremove( targetList, index )
					lastmodList[PID] = nil
					notFound = false
				end
				index = index + 1
			end
		elseif ( currentToken == "PLRMV" ) then
			-- payload will be { PID, Direction }
			SKG_Debug( "PLRMV" )
			local PID, Direction = unpack( currentPayload, 1, 2 )

			-- figure out "Position"
			local Position = 0
			for i = 1, #targetList do
				if ( targetList[i] == PID ) then
					Position = i
				end
			end

			if ( Position == 0 ) then
				-- This should never happen, but check anyway
			else
				if ( Direction == "UP") then
					targetList[Position], targetList[Position - 1] =
						targetList[Position - 1], targetList[Position]
				else
					targetList[Position], targetList[Position + 1] =
						targetList[Position + 1], targetList[Position]
				end
			end
		elseif ( currentToken == "PLRSC" ) then
			-- payload is { Player, PIDList }
			SKG_Debug( "PLRSC" )

			local Player, PIDList = unpack( currentPayload, 1, 2 )
			local moveList = {}
			local lIndex = 1
			local foundFirst = false
			for i = 1, #targetList do
				local j = 1
				local notfound = true
				while ( ( j <= #PIDList ) and ( notfound == true ) ) do
					if ( targetList[i] == PIDList[j] ) then
						notfound = false
					else
						j = j + 1
					end
				end
				if ( notfound == false ) then
					if ( targetList[i] == Player ) then
						foundFirst = true
					end
					if ( foundFirst == true ) then
						moveList[lIndex] = i
						lIndex = lIndex + 1
					end
				end
			end

			local moveLen = #moveList
			local SuicideTemp = targetList[moveList[1]]
			local moveIndex = 1
			while ( moveIndex < moveLen ) do
				local movingPID = targetList[moveList[moveIndex + 1]]
				targetList[moveList[moveIndex]] = movingPID
				moveIndex = moveIndex + 1
				lastmodList[movingPID] = currentLM
			end
			targetList[moveList[moveIndex]] = SuicideTemp
			lastmodList[Player] = currentLM
		else
			-- Don't have code to deal with this token. Report:
			SKG_Debug( "In RunTransactions, got bad token: " .. currentToken )
		end
	end
end

function SKG_tblCopy ( inT ) 
	local outT = {}
	for k, v in pairs( inT ) do
		if ( type( v ) == "table" ) then
			outT[k] = SKG_tblCopy( v )
		else
			outT[k] = v
		end
	end
	return outT
end

function SKG_ImportPhase1 ()
 	SKG_Print( SKGL["SKG_MSG_IMPORT_STARTPHASE1"] )
	SKG.Import = {}
	SKG.Import.Players = {}
	SKG.Import.RollLists = {}

	-- Pass 1 : class lists
	for listName, rList in pairs( SavedRollLists ) do
		-- if its a class list, start inserting people into
		-- the Player's List
		if ( ( listName == "Warrior" ) or ( listName == "Hunter" ) or
			 ( listName == "Shaman" ) or ( listName == "Mage" ) or
			 ( listName == "Priest" ) or ( listName == "Warlock" ) or
			 ( listName == "Rogue" ) or ( listName == "Paladin" ) or
			 ( listName == "Druid" ) ) then
			for i = 1, #rList do
				local pClass = strupper( listName )
				local pName = rList[i].player
				local PID = SKG_HashName( pName )
				local t = { 
					name = pName,
					class = pClass
				}
				SKG.Import.Players[PID] = t
			end
		else
			-- only process class lists for now. We'll process Roll Lists on the seocnd pass
		end
	end

	-- I'm going to temporarily store guild info into a struct so I can compare
	-- the toons we're importing against the guild (the most common place those
	-- toons we're not sure about will be )
	local GuildRoster = {}
	local numGuildMembers = GetNumGuildMembers(true)
	for index = 1, numGuildMembers do
		local toonName, b, b, b, toonClass = GetGuildRosterInfo(index);
		local PID = SKG_HashName( toonName )
		GuildRoster[PID] = strupper( toonClass )
	end

	-- Pass 2 : roll lists
	for listName, rList in pairs( SavedRollLists ) do
		-- if its a class list, start inserting people into
		-- the Player's List
		if ( ( listName == "Warrior" ) or ( listName == "Hunter" ) or
			 ( listName == "Shaman" ) or ( listName == "Mage" ) or
			 ( listName == "Priest" ) or ( listName == "Warlock" ) or
			 ( listName == "Rogue" ) or ( listName == "Paladin" ) or
			 ( listName == "Druid" ) ) then
			-- do nothing, we handled that in pass 1
		else
			-- its a roll list
			SKG.Import.RollLists[listName] = {}
			for i = 1, #rList do
				local pName = rList[i].player
				local PID = SKG_HashName( pName ) 
				-- make the entry in the Roll List
				SKG.Import.RollLists[listName][i] = PID
			
				-- Check to see if the record exists in our Player's list
				if ( SKG.Import.Players[PID] ) then
					-- we're gtg.
				else
					-- need to make a stub entry. I'll get filled in during Phase 2
					-- check if the class is defined in the guild...
					local t = {}
					if ( GuildRoster[PID] ) then
						t = {
							name = pName,
							class = GuildRoster[PID]
						}
					else
						t = { 
							name = pName,
							class = ""
						}
					end
					SKG.Import.Players[PID] = t
				end
			end
		end
	end
	GuildRoster = {}
	SKG_Print( SKGL["SKG_MSG_IMPORT_ENDPHASE1"] )
	SKG_ImportTimerFrame:Show()
end

function SKG_CleanUpImport( ListName )
	SKG_Debug( "ForceLastMod " .. ListName )
	local db = SKG.db.profile
	local rList = db.RollLists[ListName]
	
	local newTID = SKG_CreateTID( 0, 0, 0 )
	local timeStamp = strsub( newTID, 1, 14 )
	-- if it doesn't exist, make it
	db.LastMod[ListName] = {}
	for i = 1, #rList do
		PID = rList[i]
		db.LastMod[ListName][PID] = timeStamp
	end		

	db.Snapshot[ListName] = {}
	db.Snapshot[ListName] = rList
	db.Log[ListName] = {}
	db.SnapshotLM[ListName] = {}
	db.SnapshotLM[ListName] = db.LastMod[ListName]

	local LH = SKG_HashRollList( newTID, ListName )
	SKG_AddLogEntry( newTID, ListName, "SYNSS", LH, LH )
end

-- function SKG_ParseItem( itemLink, infoString )
--
-- This functions takes an itemLink as an argument, and returns any number of
-- data points about the item back. 

-- I don't use this function anymore. I'm leaving the stub here for documentation
-- and for the function stubs for when I do call it elsewhere, so I don't have to
-- go looking for the information again.
--
-- |cff9d9d9d|Hitem:7073:0:0:0:0:0:0:0|h[Broken Fang]|h|r
--
-- 	"|%w+|Hitem:%d+:%d+:%d+:%d+:%d+:%d+:%-?%d+:%-?%d+|h%[.-%]|h|r"
-- item:itemId:enchantId:jewelId1:jewelId2:jewelId3:jewelId4:suffixId:uniqueId
-- local itemName, _, itemRarity, itemLevel, itemMinLevel, itemType,
--		itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo( itemLink )
--
-- end

function SKG_SmartCompareItemlinks( item1, item2 )
	-- My tests were able to show that successive looting attempts on the
	-- same item from the same corpse can actually produce DIFFERENT itemStrings.
	-- In all cases, the only field that changes was the uniqueid field, and in
	-- every loot attempt AFTER the first loot attempt, the unique had high-order
	-- bytes changes
	--		One example:
	--			first loot			: 37250 ( 0x00009182 )
	--			successive loots	: 1080529282 ( 0x40679182 )
	--
	-- So in order to do a real comparison, I have to do a more thorough comparison
	-- that also includes comparing just the low-order bytes of the uniqueid.
	-- 
	-- What a pain in the ass.
	--
	-- This function will return true if the itemlinks are the same, false otherwise.
	--

	-- First, try the easy way out
	if ( item1 == item2 ) then
		return true
	else
		-- redoing using patterns. Xinhuan, you bring hell to my doorstep.
		--	|cff9d9d9d|Hitem:7073:0:0:0:0:0:0:0|h[Broken Fang]|h|r
		local i1p1, i1UID, i1p2 = string.match( item1, 
			"|%w+|Hitem:(%d+:%d+:%d+:%d+:%d+:%d+:%-?%d+:)(%-?%d+)|h%[(.-)%]|h|r" )
		local i2p1, i2UID, i2p2 = string.match( item1,
			"|%w+|Hitem:(%d+:%d+:%d+:%d+:%d+:%d+:%-?%d+:)(%-?%d+)|h%[(.-)%]|h|r" )
		
		if ( ( i1p1 == i2p1 ) and ( i1p2 == i2p2 ) ) then
			i1UID = i1UID % 65536
			i2UID = i2UID % 65536

			if ( i1UID == i2UID ) then
				return true
			end
		end
	end
	return false
end

function SKG_OpenToBidFrame()
	-- Hide all frames
	SKG_Frame:Hide()
	SKG_OptionsFrame:Hide()
	SKG_HelpFrame:Hide()
	SKG_BidFrame:Hide()
	SKG_BidButtonFrame:Hide()
	SKG_ListFrame:Hide()
	SKG_SyncFrame:Hide()
	SKG_SecurityFrame:Hide()
	-- Set the state correctly
	SKG_FRAME_STATE = "Bid"
	-- Show the right frames
	SKG_BidFrame:Show()
	SKG_Frame:Show()
	-- LOOTI handler will take care of the side frame as needed
end